/* Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "apr.h"
#include "apu.h"
#include "apr_network_io.h"

#ifndef APR_LDAP_PRIVATE_H
#define APR_LDAP_PRIVATE_H

#ifdef __cplusplus
extern "C" {
#endif

#if APR_HAS_LDAP

typedef struct apr_ldap_t {
    apr_pool_t *pool;
    LDAP *ld;
    apr_socket_t *socket;
} apr_ldap_t;

typedef struct apr_ldap_message_t {
    apr_ldap_t *ld;
    const char *rmech;
    LDAPMessage *message;
    int msgid;
} apr_ldap_message_t;

APU_DECLARE_LDAP(apr_status_t) apr_ldap_status(int rc, apr_status_t status);

APU_DECLARE_LDAP(apr_status_t) apr_ldap_message_clear(apr_ldap_message_t *msg);

APU_DECLARE_LDAP(apr_ldap_message_t *) apr_ldap_message_make(apr_ldap_t *ldap);

#if APU_DSO_BUILD

/* For LDAP internal builds, wrap our LDAP namespace */

struct apr__ldap_dso_fntable {
    int (*info)(apr_pool_t *pool, apr_ldap_err_t **result_err);
    int (*init)(apr_pool_t *pool, LDAP **ldap, const char *hostname,
                int portno, int secure, apr_ldap_err_t **result_err);
    int (*ssl_init)(apr_pool_t *pool, const char *cert_auth_file,
                    int cert_file_type, apr_ldap_err_t **result_err);
    int (*ssl_deinit)(void);
    int (*get_option)(apr_pool_t *pool, LDAP *ldap, int option,
                      void *outvalue, apr_ldap_err_t **result_err);
    int (*set_option)(apr_pool_t *pool, LDAP *ldap, int option,
                      const void *invalue, apr_ldap_err_t **result_err);
    apr_status_t (*rebind_init)(apr_pool_t *pool);
    apr_status_t (*rebind_add)(apr_pool_t *pool, LDAP *ld,
                               const char *bindDN, const char *bindPW);
    apr_status_t (*rebind_remove)(LDAP *ld);
    apr_status_t (*initialize)(apr_pool_t *pool, const char *uri,
                               apr_ldap_t **ldap,
                               apu_err_t *result_err);
    apr_status_t (*get_option_ex)(apr_ldap_t *ldap, int option,
                      apr_ldap_opt_t *outvalue, apu_err_t *result_err);
    apr_status_t (*set_option_ex)(apr_ldap_t *ldap, int option,
                      const apr_ldap_opt_t *invalue, apu_err_t *result_err);
    apr_status_t (*result)(apr_ldap_message_t *msg, apr_interval_time_t timeout,
                           apu_err_t *result);
    apr_status_t (*bind)(apr_ldap_t *ldap, const char *dn, const char *mech,
                         apr_ldap_interact_proc *interact, void *ctx,
                         apr_interval_time_t timeout,
                         apr_ldap_message_t **msg, apu_err_t *result);
};

#endif /* APU_DSO_BUILD */

#endif /* APR_HAS_LDAP */

#ifdef __cplusplus
}
#endif

#endif /* APR_LDAP_PRIVATE_H */


